@@ -2,8 +2,10 @@
 # coding: utf-8
 
 import os
+import re
 import sys
 import subprocess
+from datetime import date
 from argparse import ArgumentParser
 from pathlib import Path
 from distutils.core import run_setup
@@ -17,16 +19,28 @@ class RepositoryNotReadForReleaseError(ValueError):
     pass
 
 
-def _compute_release_tags():
+def _compute_release_tags(revset_query):
     commit_desc_since_last_release = (
         subprocess.check_output(
-            f"hg log -r '{REVSET_QUERY}' " "-T '{desc}\n'",
+            f"hg log -r '{revset_query}' " "-T '{desc}\n'",
             shell=True,
         )
         .decode("utf-8")
         .casefold()
     )
 
+    current_branch = subprocess.check_output('hg log -T "{branch}" -r .', shell=True)
+
+    # if we are already on a branch for a specific release
+    if re.match(b"^[0-9.]+$", current_branch.strip()):
+        number_of_dots = len(re.findall(rb"(\.)", current_branch.strip()))
+
+        # apparently we only have "x.y" branches for now on cubicweb and jsonschema
+        if number_of_dots == 1:  # x.y
+            return "patch"
+
+        # we'll find out on how to handle other situations later
+
     # XXX this may be a little bit too naive. But let's make things simple for a
     # first implementation.
 
@@ -50,8 +64,14 @@ def main():
     parser.add_argument(
         "--release-type",
         "-r",
-        choices=["patch", "minor", "major", "auto"],
+        choices=["patch", "minor", "major", "auto", "calendar"],
         default="auto",
+        help=(
+            "Use either\n:"
+            " - semantic version: major.minor.patch \n"
+            " - calendar version: YY.M.num where num is the version number in the month \n"
+            "auto use semantic versionning and detect the correct version from commit messages."
+        ),
     )
     parser.add_argument("--no-previous-tag-check", action="store_true")
     parser.add_argument(
@@ -72,6 +92,11 @@ def main():
         help="the mercurial revset query to get the commit used to generate the changelog",
         default=REVSET_QUERY,
     )
+    parser.add_argument(
+        "-n",
+        "--version-number",
+        help="explicitly ask for a version number",
+    )
 
     args = parser.parse_args()
 
@@ -94,6 +119,7 @@ def main():
             args.no_previous_tag_check,
             dry_run=args.dry_run,
             revset_query=args.revset_query,
+            version_number=args.version_number,
         )
     except RepositoryNotReadForReleaseError as exception:
         # kdo Arthur
@@ -101,12 +127,9 @@ def main():
         sys.exit(1)
 
 
-def do_release(release_type, no_previous_tag_check, dry_run, revset_query):
-    if release_type == "auto":
-        effective_release_type = _compute_release_tags()
-    else:
-        effective_release_type = release_type
-
+def do_release(
+    release_type, no_previous_tag_check, dry_run, revset_query, version_number
+):
     root = Path(".")
 
     # assert there is a pkginfo and a setup.py
@@ -179,7 +202,7 @@ def do_release(release_type, no_previous_tag_check, dry_run, revset_query):
     )
 
     if not any(current_version in tag for tag in existing_tags):
-        if not no_previous_tag_check:
+        if not no_previous_tag_check and not version_number:
             raise RepositoryNotReadForReleaseError(
                 "cannot handle this situation right now. "
                 f"Your current version ({current_version}) is not found in "
@@ -189,47 +212,81 @@ def do_release(release_type, no_previous_tag_check, dry_run, revset_query):
 
     # should we check the version against pypi ?
 
-    # exist, warn and exit
-    # ask for :
-    #  - patch
-    #  - minor
-    #  - major
+    if version_number:
+        red = RedBaron(pkginfo_path.read_text())
+        assignement = red.find("assign", target=lambda x: x.value == "numversion")
+        assert assignement
 
-    print(f"Let's go for a {effective_release_type} release")
+        old_version = ".".join(str(x.value) for x in assignement.value)
 
-    red = RedBaron(pkginfo_path.read_text())
-    assignement = red.find("assign", target=lambda x: x.value == "numversion")
-    assert assignement
-
-    if effective_release_type == "patch":
-        assignement.value[2].value = str(int(assignement.value[2].value) + 1)
-
-    elif effective_release_type == "minor":
-        assignement.value[2].value = "0"
-        assignement.value[1].value = str(int(assignement.value[1].value) + 1)
-
-    elif effective_release_type == "major":
-        assignement.value[2].value = "0"
-        assignement.value[1].value = "0"
-        assignement.value[0].value = str(int(assignement.value[0].value) + 1)
+        assignement.value = f'({", ".join(version_number.split("."))})'
 
+        effective_release_type = "fix-by-user"
+        new_version = version_number
     else:
-        raise Exception("unhandled situation")
+        if release_type == "auto":
+            effective_release_type = _compute_release_tags(revset_query)
+        else:
+            effective_release_type = release_type
+
+        red = RedBaron(pkginfo_path.read_text())
+        assignement = red.find("assign", target=lambda x: x.value == "numversion")
+        assert assignement
+
+        major = int(assignement.value[0].value)
+        minor = int(assignement.value[1].value)
+        patch = int(assignement.value[2].value)
+
+        old_version = ".".join(str(x.value) for x in assignement.value)
+
+        if effective_release_type == "patch":
+            assignement.value[2].value = str(patch + 1)
+
+        elif effective_release_type == "minor":
+            assignement.value[2].value = "0"
+            assignement.value[1].value = str(minor + 1)
+
+        elif effective_release_type == "major":
+            assignement.value[2].value = "0"
+            assignement.value[1].value = "0"
+            assignement.value[0].value = str(major + 1)
+
+        elif effective_release_type == "calendar":
+            today = date.today()
+            year = today.strftime("%y")
+            month = str(today.month)
+            new_patch = "0"
+            if f"{major}.{minor}" == f"{year}.{month}":
+                new_patch = str(patch + 1)
+            assignement.value[2].value = new_patch
+            assignement.value[1].value = month
+            assignement.value[0].value = year
+
+        else:
+            raise Exception("unhandled situation")
+
+        new_version = ".".join(str(x.value) for x in assignement.value)
 
     pkginfo_path.write_text(red.dumps())
 
-    new_version = ".".join(str(x.value) for x in assignement.value)
+    print(
+        f"Let's go for {effective_release_type} release {new_version} (from {old_version})"
+    )
 
-    if release_type == "auto":
-        print(
-            f"Automatic release guesser decided to release the version '{new_version}' "
-            f"({effective_release_type})"
-        )
-        answer = input("Are you ok with that? [Y/n]: ")
+    if not version_number:
+        if release_type in ["auto", "calendar"]:
+            msg = "Automatic"
+            if release_type == "calendar":
+                msg = "Calendar"
+            print(
+                f"{msg} release guesser decided to release the version '{new_version}' "
+                f"({effective_release_type})"
+            )
+            answer = input("Are you ok with that? [Y/n]: ")
 
-        if answer.strip().lower() == "n":
-            subprocess.check_call("hg revert .", shell=True)
-            return
+            if answer.strip().lower() == "n":
+                subprocess.check_call("hg revert .", shell=True)
+                return
 
     changelog_path = Path("CHANGELOG.md")
     changelog_format = "md"
@@ -287,19 +344,28 @@ def do_release(release_type, no_previous_tag_check, dry_run, revset_query):
     subprocess.check_call(f"hg add '{changelog_path}'", shell=True)
 
     if not dry_run:
-        subprocess.check_call(
-            f'hg commit -m "chore(pkg): new {effective_release_type} release ({new_version})"',
-            shell=True,
-        )
+        if not version_number:
+            subprocess.check_call(
+                f'hg commit -m "chore(pkg): new {effective_release_type} release ({new_version})"',
+                shell=True,
+            )
+        else:
+            subprocess.check_call(
+                f'hg commit -m "chore(pkg): new release ({new_version})"',
+                shell=True,
+            )
 
         subprocess.check_call(f'hg tag "{new_version}"', shell=True)
         subprocess.check_call("hg phase -p .", shell=True)
 
         # emojis for Arthur
-        print(
-            f"🎉 Congratulation, we've made a new {effective_release_type} release {new_version} "
-            "\\o/ 🎇"
-        )
+        if not version_number:
+            print(
+                f"🎉 Congratulation, we've made a new {effective_release_type} release "
+                f"{new_version} \\o/ 🎇"
+            )
+        else:
+            print(f"🎉 Congratulation, we've made a new release {new_version} " "\\o/ 🎇")
         print()
         print("✨ 🍰 ✨")
         print()
